<?php

/**
 * Plugin Name:       ZhuiGe User Dummy
 * Plugin URI:        https://gitee.com/longwenjunj/zhuige-user-dummy
 * Description:       批量生成马甲用户，适用于酱茄-追格WordPress系列小程序、主题。
 * Version:           1.0.0
 * Author:            追格
 * Author URI:        https://www.zhuige.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

if (!defined('WPINC')) {
	die;
}

define('JQ_USER_DUMMY_VERSION', '1.0.0');
define('JQ_USER_DUMMY_BASE_NAME', plugin_basename(__FILE__));
define('JQ_USER_DUMMY_BASE_DIR', plugin_dir_path(__FILE__));
define('JQ_USER_DUMMY_BASE_URL', plugin_dir_url(__FILE__));

define('JQ_USER_DUMMY_NICKNAME_FILE', JQ_USER_DUMMY_BASE_DIR . 'nickname.txt');
define('JQ_USER_DUMMY_AVATAR_DIR', JQ_USER_DUMMY_BASE_DIR . 'avatar');

/**
 * 启用与禁用
 */
function activate_zhuige_user_dummy()
{
	require_once JQ_USER_DUMMY_BASE_DIR . 'includes/class-zhuige-user-dummy-activator.php';
	ZhuiGe_User_Dummy_Activator::activate();
}

function deactivate_zhuige_user_dummy()
{
	require_once JQ_USER_DUMMY_BASE_DIR . 'includes/class-zhuige-user-dummy-deactivator.php';
	ZhuiGe_User_Dummy_Deactivator::deactivate();
}

register_activation_hook(__FILE__, 'activate_zhuige_user_dummy');
register_deactivation_hook(__FILE__, 'deactivate_zhuige_user_dummy');

/**
 * 设置链接
 */
function zhuige_user_dummy_action_links($actions)
{
	$actions[] = '<a href="admin.php?page=zhuige_user_dummy">设置</a>';
	$actions[] = '<a href="https://www.zhuige.com/" target="_blank">技术支持</a>';
	return $actions;
}
add_filter('plugin_action_links_' . JQ_USER_DUMMY_BASE_NAME, 'zhuige_user_dummy_action_links');

require JQ_USER_DUMMY_BASE_DIR . 'includes/class-zhuige-user-dummy.php';
require JQ_USER_DUMMY_BASE_DIR . 'includes/zhuige-user-dummy-dashboard.php';

/**
 * 用户头像
 */
add_filter('get_avatar_url', 'zhuige_user_dummy_get_avatar_url', 10, 2);
function zhuige_user_dummy_get_avatar_url($url, $id_or_email, $args = null)
{
	$options = get_option('zhuige_dummy_setting');
	if (empty($options['avatar_meta_key'])) {
		return $url;
	}

	$jq_avatar = get_user_meta($id_or_email, $options['avatar_meta_key'], true);
	if ($jq_avatar) {
		return $jq_avatar;
	} else {
		return $url;
	}
}

/**
 * 读取头像文件
 */
function zhuige_user_dummy_avatars($dir)
{
	if (!is_dir($dir)) {
		return [1];
	}

	$dh = opendir($dir);
	if (!$dh) {
		return [];
	}

	$avatars = [];
	while (($filename = readdir($dh)) !== false) {
		if ($filename == "." || $filename == "..") {
			continue;
		}

		$recursiveDir = $dir . DIRECTORY_SEPARATOR . $filename;
		if (is_dir($recursiveDir)) {
			$avatars = array_merge($avatars, zhuige_user_dummy_avatars($recursiveDir));
		} else {
			$avatars[] = plugins_url($filename, $recursiveDir);
		}
	}

	closedir($dh);

	return $avatars;
}

function zhuige_user_dummy_file_content($file) {
	return (file_exists($file) ? file_get_contents($file) : '');
}

/**
 * 生成预览
 */
function zhuige_user_dummy_pre_create($avatar_start=false, $nick_start=false)
{
	$options = get_option('zhuige_dummy_setting');
	$count = (int)($options['dummy_once_count']);
	$suffix = $options['nickname_suffix'];

	$avatars = zhuige_user_dummy_avatars(JQ_USER_DUMMY_AVATAR_DIR);
	$avatar_count = count($avatars);
	if (!$avatar_start) {
		$avatar_start = rand(0, $avatar_count - 1);
	}
	$nick_str = zhuige_user_dummy_file_content(JQ_USER_DUMMY_NICKNAME_FILE);
	$nick_arr = explode(',', $nick_str);
	$nicks = [];
	foreach ($nick_arr as $nick) {
		if (empty($nick)) {
			continue;
		}
		$nicks[] = $nick;
	}
	$nick_count = count($nicks);
	if (!$nick_start) {
		$nick_start = rand(0, $nick_count - 1);
	}

	$users = get_users();
	$old_nicks = [];
	foreach ($users as $user) {
		$old_nicks[] = get_user_meta($user->id, 'nickname', true);
	}
	
	$list = [];
	for ($i = 0; $i < $count; $i++) {
		$avatar = $avatars[($i + $avatar_start) % $avatar_count];
		$nick = $nicks[($i + $nick_start) % $nick_count];

		$new_nick = $nick;
		if ($suffix != '0') {
			$no = 2;
			while (in_array($new_nick, $old_nicks)) {
				if ($suffix == '1') {
					$new_nick = $nick . $no;
				} else if ($suffix == '2') {
					$new_nick = $nick . 'No.' . $no;
				} else if ($suffix == '3') {
					$new_nick = $nick . $no . '号';
				} else {
					$nickname_suffix_custom = $options['nickname_suffix_custom'];
					if (strpos($nickname_suffix_custom, '%d') !== false) {
						$new_nick = sprintf("%s" . $nickname_suffix_custom, $nick, $no);
					} else {
						$new_nick = $nick . $no;
					}
				}

				$no++;
			}
			$old_nicks[] = $new_nick;
		}

		if (empty($options['avatar_meta_key'])) {
			$avatar = '';
		}

		$list[] = [
			'nick' => $new_nick,
			'avatar' => $avatar,
		];
	}

	return[
		'nick_start' => $nick_start,
		'avatar_start' => $avatar_start,
		'list' => $list,
	];
}

function run_zhuige_user_dummy()
{
	$plugin = new ZhuiGe_User_Dummy();
	$plugin->run();
}
run_zhuige_user_dummy();
